/*
 * Copyright 2015 Freescale Semiconductor, Inc.
 * Copyright 2015-2021 NXP.
 *
 * License: LA_OPT_NXP_Software_License
 *
 * This software is owned or controlled by NXP and may
 * only be used strictly in accordance with the applicable license terms.
 * By expressly accepting such terms or by downloading, installing,
 * activating and/or otherwise using the software, you are agreeing that
 * you have read, and that you agree to comply with and are bound by,
 * such license terms.  If you do not agree to be bound by the applicable
 * license terms, then you may not retain, install, activate or otherwise
 * use the software.
 * 
 * @version   4.1
 *
 * @brief     IEC60730 Class B Safety Routines Flash tests for DSP56800EX core
 *
 */

#ifndef _IEC60730B_DSC_FLASH_
#define _IEC60730B_DSC_FLASH_


/******************************************************************************
* Functions
******************************************************************************/
#ifndef __ASM__


/*******************************************************************************
*
* @brief  This function generate the 16-bit CRC value using HW CRC module /CRC-CCITT. 
*         Clocks to the CRC module must be enabled prior using this function. 
*         Can be used only for program memory addresses.  
*
* @param *startAddress  - first address of the tested memory
* @param size           - size of tested memory
* @param *moduleAddress - address of CRC module	  
* @param crcVal         - start condition seed
*
* @return 16-bit crc value of the memory range (CRC-16-CCITT - normal 0x1021)
*
*******************************************************************************/
unsigned short FS_flash_HW16_program(unsigned long *startAddress, unsigned long size, unsigned long *moduleAddress, unsigned short crcVal);

/*******************************************************************************
*
* @brief  This function generate the 16-bit CRC value using HW CRC module /CRC-CCITT. 
*         Clocks to the CRC module must be enabled prior using this function. 
*         Can be used only for data memory addresses. 
*
* @param *startAddress  - first address of the tested memory
* @param size           - size of tested memory
* @param *moduleAddress - address of CRC module	  
* @param crcVal         - start condition seed
*
* return 16-bit crc value of the memory range (CRC-16-CCITT - normal 0x1021)
*		
*******************************************************************************/
unsigned short FS_flash_HW16_data(unsigned long *startAddress, unsigned long size, unsigned long *moduleAddress, unsigned short crcVal);

/*******************************************************************************
*
* @brief  This function generate the 16-bit CRC value using the software calculation /CRC-CCITT. 
*         Can be used only for program memory addresses. 
*
* @param *startAddress  - first address of the tested memory
* @param size           - size of tested memory
* @param *moduleAddress - address of CRC module	  
* @param crcVal         - start condition seed
*
* @return 16-bit crc value of the memory range (CRC-16-CCITT - normal 0x1021)
*		
*******************************************************************************/
unsigned short FS_flash_SW16_program(unsigned long *startAddress, unsigned long size, unsigned long *moduleAddress, unsigned short crcVal);

/*******************************************************************************
*
* @brief  This function generate the 16-bit CRC value using software calculation /CRC-CCITT. 
*        Can be used only for data memory addresses.  
*
* @param *startAddress  - first address of the tested memory
* @param size           - size of tested memory
* @param *moduleAddress - address of CRC module	  
* @param crcVal         - start condition seed
*
* @return 16-bit crc value of the memory range (CRC-16-CCITT - normal 0x1021)
*
*******************************************************************************/
unsigned short FS_flash_SW16_data(unsigned long *startAddress, unsigned long size, unsigned long *moduleAddress, unsigned short crcVal);


#endif /* __ASM__ */

#endif /* _IEC60730B_DSC_FLASH_ */
