/*
 * Copyright 2015 Freescale Semiconductor, Inc.
 * Copyright 2015-2021 NXP.
 *
 * License: LA_OPT_NXP_Software_License
 *
 * This software is owned or controlled by NXP and may
 * only be used strictly in accordance with the applicable license terms.
 * By expressly accepting such terms or by downloading, installing,
 * activating and/or otherwise using the software, you are agreeing that
 * you have read, and that you agree to comply with and are bound by,
 * such license terms.  If you do not agree to be bound by the applicable
 * license terms, then you may not retain, install, activate or otherwise
 * use the software.
 * 
 * @version   4.1
 * 
 *
 * @brief     IEC60730 Class B Safety Routines Program counter tests for DSP56800EX core
 *
 */

#ifndef _IEC60730B_DSC_PC_
#define _IEC60730B_DSC_PC_


#ifndef __ASM__

/******************************************************************************
* Definitions
******************************************************************************/
typedef void (*tFcn_pc)(void);

/******************************************************************************
* Functions
******************************************************************************/
/*********************************************************************************
*
* @brief  Function checks the PC register. It jumps to the object function and also to the subroutine.
*         Object functions should be placed at an address that is used as the pattern for PC register.
*         Second parameter, pSubroutine must always be  FS_PC_Subroutine.
*         *PC_flag points to the flag that is set at the begin of the function and cleared
*         after a successful execution
*
* @param  pObject_function  - pointer to an object function
* @param  pSubroutine       - pointer to the subroutine. Fixed parameter, must always be the same.
* @param  *PC_flag          - pointer to a flag that is set at the begining and clear at the end of the function
*
* @return FS_RESULT
*		 
************************************************************************************/
FS_RESULT FS_PC_Test(tFcn_pc pObject_function, tFcn_pc pSubroutine, unsigned short* PC_flag);

/*********************************************************************************
*
* @brief  Subroutine that is used as an auxiliary function for PC test
*
* @param  none
* 
* @return void
*		 
************************************************************************************/
void FS_PC_subroutine(void);

/*********************************************************************************
*
* @brief  An object that can be placed at a suitable address by linker
*
* @param  none
* 
* @return void
*		 
************************************************************************************/
void FS_PC_object_1(void);

/*********************************************************************************
*
* @brief  An object that can be placed at a suitable address by linker
*
* @param  none
* 
* @return void
*		 
************************************************************************************/
void FS_PC_object_2(void);




#endif /* __ASM__ */

#endif /* _IEC60730B_DSC_PC_ */
