/*
 * Copyright 2015 Freescale Semiconductor, Inc.
 * Copyright 2015-2021 NXP.
 *
 * License: LA_OPT_NXP_Software_License
 *
 * This software is owned or controlled by NXP and may
 * only be used strictly in accordance with the applicable license terms.
 * By expressly accepting such terms or by downloading, installing,
 * activating and/or otherwise using the software, you are agreeing that
 * you have read, and that you agree to comply with and are bound by,
 * such license terms.  If you do not agree to be bound by the applicable
 * license terms, then you may not retain, install, activate or otherwise
 * use the software.
 * 
 * @version   4.1
 *
 * @brief     IEC60730 Class B Safety Routines RAM tests for DSP56800EX core
 *
 */

#ifndef _IEC60730B_DSC_RAM_
#define _IEC60730B_DSC_RAM_



#ifndef __ASM__

/******************************************************************************
* Definitions
******************************************************************************/
typedef FS_RESULT (*tFcn_ram)(unsigned long*, unsigned short);

/******************************************************************************
* Functions
******************************************************************************/
/*******************************************************************************
*
* @brief  Function performs March test of the RAM , after reset.
*
* @param  *startAddress      first address of the tested memory 
*         *backup_address    address of the dedicated memory where data from tested memory are backed up
*         pMarch_type        pointer to the March function from the library. Specifies which march pattern is used
*         block_size         size of the test iteration.  Cannot be larger than the size of backup memory.
*         test_range         size of the tested memory. 
*
* @return  FS_RAM_PASS  - Test successfully done
*          FS_RAM_FAIL  - Test failed
*
*******************************************************************************/
FS_RESULT FS_RAM_AfterReset(unsigned long *start_address, unsigned long *backup_address, tFcn_ram pMarch_type, unsigned short block_size, unsigned short test_range);

/*******************************************************************************
*
* @brief  Function performs March test of the RAM , during runtime.
*
* @param  *startAddress     first address of the tested memory 
*         *backup_address   address of the dedicated memory where data from tested memory are backed up
*         pMarch_type      - pointer to the March function from the library. Specifies which march pattern is used
*         *actual_address  - pointer to an actual address of the test. In initialisation set to the startAddress value. During runtime updated by the function itself. 
*         block_size       - size of the test iteration.  Cannot be larger than the size of backup memory.
*         test_range        size of the tested memory. 
*
* @return   FS_RAM_PASS  - Test successfully done
*           FS_RAM_FAIL  - Test failed
*
*******************************************************************************/
FS_RESULT FS_RAM_Runtime(unsigned long *start_address, unsigned long *backup_address, tFcn_ram pMarch_type, unsigned long *actual_address, unsigned short block_size, unsigned short test_range);

/*******************************************************************************
*
* @brief  Function copy block of memory from source address to destination address.
*
* @param  *pMemSource - source address
*         *pMemDest   - destination address
*         uintLength  - size of the block to be copied 
*
* @return void
*
*******************************************************************************/
void FS_RAM_CopyMemory(unsigned long *pMemSource, unsigned long *pMemDest, unsigned short uintLength);

/*******************************************************************************
*
* @brief  Function performs March C test.
*
* @param  *start_address - first address to perform march test on
*         block_size     - size of the memory to test
*
* @return  FS_RAM_PASS  - Test successfully done
*          FS_RAM_FAIL  - Test failed
*
*******************************************************************************/
FS_RESULT FS_RAM_SegmentMarchC(unsigned long *start_address, unsigned short block_size);

/*******************************************************************************
*
* @brief  Function performs March X test.
*
* @param  *start_address - first address to perform march test on
*         block_size     - size of the memory to test
*
* @return  FS_RAM_PASS  - Test successfully done
*          FS_RAM_FAIL  - Test failed
*
*******************************************************************************/
FS_RESULT FS_RAM_SegmentMarchX(unsigned long *start_address, unsigned short block_size);



#endif /* __ASM__ */

#endif /* _IEC60730B_DSC_RAM_ */
