/*
 * Copyright 2015 Freescale Semiconductor, Inc.
 * Copyright 2015-2021 NXP.
 *
 * License: LA_OPT_NXP_Software_License
 *
 * This software is owned or controlled by NXP and may
 * only be used strictly in accordance with the applicable license terms.
 * By expressly accepting such terms or by downloading, installing,
 * activating and/or otherwise using the software, you are agreeing that
 * you have read, and that you agree to comply with and are bound by,
 * such license terms.  If you do not agree to be bound by the applicable
 * license terms, then you may not retain, install, activate or otherwise
 * use the software.
 * 
 * @version   4.1 
 *
 * @brief     IEC60730 Class B Safety Routines CPU tests for DSP56800EX core
 *
 */

#ifndef _IEC60730B_DSC_REG_
#define _IEC60730B_DSC_REG_

/*******************************************************************************
* Macros
*******************************************************************************/
/* Registers test result constants */
#define REG_OK         0
#define REG_FAIL_X0    3
#define REG_FAIL_A     4
#define REG_FAIL_B     5
#define REG_FAIL_C     6
#define REG_FAIL_D     7

#define REG_FAIL_R0   15
#define REG_FAIL_R1   16
#define REG_FAIL_R2   17
#define REG_FAIL_R3   18
#define REG_FAIL_R4   19
#define REG_FAIL_R5   20
#define REG_FAIL_N    21
#define REG_FAIL_M01  22
#define REG_FAIL_N3   23

#define REG_FAIL_R0S  25
#define REG_FAIL_R1S  26
#define REG_FAIL_NS   27
#define REG_FAIL_M01S 28

#define REG_FAIL_N3S  38
#define REG_FAIL_R2S  39
#define REG_FAIL_R3S  40
#define REG_FAIL_R4S  41
#define REG_FAIL_R5S  42

#define REG_FAIL_SR   29

#define REG_FAIL_SP   24

#define REG_FAIL_LC   30
#define REG_FAIL_LA   31
#define REG_FAIL_HWS  32
#define REG_FAIL_LC2  33
#define REG_FAIL_LA2  34
#define REG_FAIL_HWS1 35


#ifndef __ASM__

/******************************************************************************
* Functions
******************************************************************************/
/********************************************************************************
 *
 * @brief  The function tests DSC ALU registers A, B, C, D, Y, X0 
 *         for stuck at fault
 * 
 * @param  none
 *
 * @return Result of test: REG_OK      - all tests passed
 *                         REG_FAIL_A  - failure of A
 *                         REG_FAIL_B  - failure of B
 *                         REG_FAIL_C  - failure of C
 *                         REG_FAIL_D  - failure of D
 *                         REG_FAIL_X0 - failure of X0
 *                         or function can stuck in the endless loop if test of Y register failed     
 * 
 ********************************************************************************/
FS_RESULT FS_CPU_DataRegisters(void);

/********************************************************************************
 *
 * @brief  The function tests DSC AGU registers A, R0, R1, R2, R3, R4, R5, N, M01, N3 
 *         for stuck at fault.
 * 
 * @param  none
 *
 * @return Result of test: REG_OK       - all tests passed
 *                         REG_FAIL_R0  - failure of R0
 *                         REG_FAIL_R1  - failure of R1
 *                         REG_FAIL_R2  - failure of R2
 *                         REG_FAIL_R3  - failure of R3
 *                         REG_FAIL_R4  - failure of R4
 *                         REG_FAIL_R5  - failure of R5
 *                         REG_FAIL_N   - failure of N
 *                         REG_FAIL_M01 - failure of M01
 *                         REG_FAIL_N3  - failure of N3     
 *                         
 ********************************************************************************/
FS_RESULT FS_CPU_PointerRegisters(void);

/********************************************************************************
 *
 * @brief  The function tests shadowed DSC AGU registers R0, R1, N, M01 that are present in both 56800E and 56800EX cores 
 *         for stuck at fault.
 * 
 * @param  none
 *
 * @return Result of test: REG_OK        - all tests passed
 *                         REG_FAIL_R0S  - failure of shadowed R0
 *                         REG_FAIL_R1S  - failure of shadowed R1
 *                         REG_FAIL_NS   - failure of shadowed N
 *                         REG_FAIL_M01S - failure of shadowed M01
 * 
 ********************************************************************************/
FS_RESULT FS_CPU_ShadowRegistersE(void);

/********************************************************************************
 *
 * @brief  The function tests shadowed DSC AGU registers R2, R3, R4, R5, N3 that are present only in 56800EX core 
 *         for stuck at fault.
 * 
 * @param  none
 *
 * @return Result of test: REG_OK        - all tests passed
 *                         REG_FAIL_R2S  - failure of shadowed R2
 *                         REG_FAIL_R3S  - failure of shadowed R3
 *                         REG_FAIL_R4S  - failure of shadowed R4
 *                         REG_FAIL_R5S  - failure of shadowed R5
 *                         REG_FAIL_N3S  - failure of shadowed N3
 * 
 ********************************************************************************/
FS_RESULT FS_CPU_ShadowRegistersEX(void);

/********************************************************************************
 *
 * @brief  The function tests DSC status register (SR)
 *         for stuck at fault.
 * 
 * @param  none
 *
 * @return Result of test: REG_OK  - all tests passed
 *                         or function can stuck in the endless loop if test failed     
 * 
 ********************************************************************************/
FS_RESULT FS_CPU_StatusRegister(void);

/********************************************************************************
 *
 * @brief  The function tests DSC stack pointer (SP)
 *         for stuck at fault.
 * 
 * @param  none
 *
 * @return Result of test: REG_OK  - all tests passed
 *                         or function can stuck in the endless loop if test failed     
 * 
 ********************************************************************************/
FS_RESULT FS_CPU_StackPointer(void);

/********************************************************************************
 *
 * @brief  The function tests DSC - Program Control Unit registers for hardware looping LC, LC2, LA, LA2, HWS, HWS1 
 *         for stuck at fault.
 * 
 * @param  none
 *
 * @return Result of test: REG_OK        - all tests passed
 *                         REG_FAIL_LC   - failure of LC
 *                         REG_FAIL_LA   - failure of LA
 *                         REG_FAIL_HWS  - failure of HWS
 *                         REG_FAIL_HWS1 - failure of HWS1
 *                         REG_FAIL_LA2  - failure of LA2
 *                         REG_FAIL_LC2  - failure of LC2
 * 
 ********************************************************************************/
FS_RESULT FS_CPU_LoopRegisters(void);


#endif /* __ASM__ */

#endif /* _IEC60730B_DSC_REG_ */
