/*
 * Copyright 2015 Freescale Semiconductor, Inc.
 * Copyright 2015-2021 NXP.
 *
 * License: LA_OPT_NXP_Software_License
 *
 * This software is owned or controlled by NXP and may
 * only be used strictly in accordance with the applicable license terms.
 * By expressly accepting such terms or by downloading, installing,
 * activating and/or otherwise using the software, you are agreeing that
 * you have read, and that you agree to comply with and are bound by,
 * such license terms.  If you do not agree to be bound by the applicable
 * license terms, then you may not retain, install, activate or otherwise
 * use the software.
 *
 * @version   4.1
 *
 * @brief     IEC60730 Class B Safety Routines stack tests for DSP56800EX core
 *
 */

#ifndef _IEC60730B_DSC_STACK_
#define _IEC60730B_DSC_STACK_


#ifndef __ASM__

/******************************************************************************
* Functions
******************************************************************************/
/*******************************************************************************
*
* @brief  Function performs initialisation of Stack test. it fills areas above and below the stack with 
*         the predefined pattern. The areas must be properly defined in Linker
*
* @param  test_pattern     - Pattern to be written into the areas
*         *first_address   - the address closest to the beginning of the stack
*         *second_address  - the address closest to the end of the stack
*         block_size       - size of areas 
*
* @return void
*
*******************************************************************************/
void FS_StackInit(unsigned short test_pattern, unsigned short *first_address,unsigned short *second_address, unsigned short block_size);

/*******************************************************************************
*
* @brief  Function performs the Stack overflow test. It checks if the patterns are still written in the defined areas.
*
* @param  test_pattern     - Pattern to be written into the areas
*         *first_address   - the address closest to the beginning of the stack
*         *second_address  - the address closest to the end of the stack
*         block_size       - size of areas 
*
* @return  FS_STACK_PASS  - Test successfully done
*          FS_STACK_FAIL  - Test failed
*
*******************************************************************************/
FS_RESULT FS_StackTest(unsigned short test_pattern, unsigned short *first_address, unsigned short *second_address, unsigned short block_size);





#endif /* __ASM__ */

#endif /* _IEC60730B_DSC_STACK_ */
